package com.yonge.cooleshow.auth.core.provider.service;

import com.yonge.cooleshow.api.feign.AdminFeignService;
import com.yonge.cooleshow.api.feign.dto.EmployeeApi;
import com.yonge.cooleshow.api.feign.dto.StudentApi;
import com.yonge.cooleshow.api.feign.dto.TeacherApi;
import com.yonge.cooleshow.api.feign.dto.TenantWrapper;
import com.yonge.cooleshow.auth.api.dto.SysUserInfo;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.auth.service.SysUserService;
import com.yonge.cooleshow.common.enums.SysUserType;
import com.yonge.cooleshow.common.enums.UserLockFlag;
import com.yonge.cooleshow.common.security.AuthUser;
import com.yonge.cooleshow.common.security.SecurityConstants;
import com.yonge.cooleshow.common.util.WebUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
public class DefaultUserDetailsService implements UserDetailsService {

    @Autowired
    private SysUserService sysUserService;

    @Resource
    private AdminFeignService adminFeignService;

    private final String BCRYPT = "{bcrypt}";

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        if (StringUtils.isBlank(username)) {
            return null;
        }
        String clientId = WebUtil.getParameter("clientId").toUpperCase();
        String clientSecret = WebUtil.getParameter("clientSecret");

        SysUserInfo userInfo;

        String openid = "";
        if (StringUtils.startsWith(username, SecurityConstants.PHONE_PRINCIPAL_PREFIX)) {
            userInfo = sysUserService.queryUserInfoByPhone(StringUtils.substringAfter(username, SecurityConstants.PHONE_PRINCIPAL_PREFIX));
        } else if (StringUtils.startsWith(username, SecurityConstants.USERNAME_PRINCIPAL_PREFIX)) {
            userInfo = sysUserService.queryUserInfoByUsername(StringUtils.substringAfter(username, SecurityConstants.USERNAME_PRINCIPAL_PREFIX));
        } else if (StringUtils.startsWith(username, SecurityConstants.MA_PRINCIPAL_PREFIX)) {
            String[] values = username.split(":");
            // 设置用户的openId
            openid = values[1];
            // 根据手机号查询用户信息
            userInfo = sysUserService.queryUserInfoByPhone(values[2]);
        } else {
            userInfo = sysUserService.queryUserInfoByUsername(username);
        }
        SysUser sysUser = userInfo.getSysUser();

        if (userInfo == null || sysUser == null) {
            throw new UsernameNotFoundException("账户不存在");
        } else if (sysUser.getLockFlag() == 9) {
            throw new LockedException("账户被锁定");
        }
        //判断用户是否被锁定
        if (SysUserType.STUDENT.getCode().equals(clientId)) {
            StudentApi data = adminFeignService.getStudent(sysUser.getId()).getData();
            if (data == null) {
                throw new UsernameNotFoundException("账户不存在");
            }
            if (UserLockFlag.LOCKED.equals(data.getLockFlag())) {
                throw new LockedException("账户被锁定");
            }
        }else if(SysUserType.TEACHER.getCode().equals(clientId) || "qr_teacher".equalsIgnoreCase(clientId)) {
            TeacherApi data = adminFeignService.getTeacher(sysUser.getId()).getData();
            if (data == null) {
                throw new UsernameNotFoundException("账户不存在");
            }
            if (UserLockFlag.LOCKED.equals(data.getLockFlag())) {
                throw new LockedException("账户被锁定");
            }
        }else if(SysUserType.SYSTEM.getCode().equals(clientId)) {
            EmployeeApi data = adminFeignService.getEmployee(sysUser.getId()).getData();
            if (data == null) {
                throw new UsernameNotFoundException("账户不存在");
            }
            if (UserLockFlag.LOCKED.equals(data.getLockFlag())) {
                throw new LockedException("账户被锁定");
            }
        } else if (SysUserType.ORGANIZATION.getCode().equals(clientId)) {
            TenantWrapper.Staff tenantStaff = adminFeignService.getTenantStaff(sysUser.getId()).getData();
            if (tenantStaff == null) {
                throw new UsernameNotFoundException("账户不存在");
            }
            if (Boolean.FALSE.equals(tenantStaff.getTenantEnableFlag())) {
                // 机构锁定
                throw new UsernameNotFoundException("MA:5004:" + openid);
            }
            if (UserLockFlag.LOCKED.name().equals(tenantStaff.getStatus())) {
                // 账号被锁定
                throw new UsernameNotFoundException("MA:5005:" + openid);
            }
        } else if ("WEBSITE".equals(clientId)) {
            String loginUserType = WebUtil.getParameter("loginUserType").toUpperCase();
            if (SysUserType.STUDENT.getCode().equals(loginUserType)) {
                StudentApi data = adminFeignService.getStudent(sysUser.getId()).getData();
                if (data == null) {
                    throw new UsernameNotFoundException("账户不存在");
                }
                if (UserLockFlag.LOCKED.equals(data.getLockFlag())) {
                    throw new LockedException("账户被锁定");
                }
            }else if(SysUserType.TEACHER.getCode().equals(loginUserType)) {
                TeacherApi data = adminFeignService.getTeacher(sysUser.getId()).getData();
                if (data == null) {
                    throw new UsernameNotFoundException("账户不存在");
                }
                if (UserLockFlag.LOCKED.equals(data.getLockFlag())) {
                    throw new LockedException("账户被锁定");
                }
            }
        }

        List<GrantedAuthority> authorities = null;

        String userType = sysUser.getUserType();
        String[] roles = userType.split(",");

        if (userType.contains("SYSTEM")) {
            authorities = AuthorityUtils.createAuthorityList(userInfo.getPermissions());
        }

        if (userType.contains("STUDENT")) {
            // 学生账号修改隐藏为显示
            sysUserService.updateStudentHideFlag(userInfo.getSysUser().getId(), 0);
        }

        if (authorities == null) {
            authorities = new ArrayList<>();
        }

        authorities.add(new SimpleGrantedAuthority(StringUtils.lowerCase(userType)));

        for (String role : roles) {
            authorities.add(new SimpleGrantedAuthority(StringUtils.lowerCase(role)));
        }

        return new AuthUser(sysUser.getId(), username, BCRYPT + sysUser.getPassword(), sysUser.getLockFlag() == 0, true, true, true, authorities, clientId, clientSecret);
    }

}
